package org.n52.security.common.collections;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Simple util for Iterable and Iterators.
 */
public class CollectionUtil {

    public static <CollectionType extends Collection<ItemType>, ItemType> CollectionType addAll(
            CollectionType col,
            Iterable<? extends ItemType> items) {
        for (ItemType item : items) {
            col.add(item);
        }
        return col;
    }

    public static <CollectionType extends Collection<ItemType>, ItemType> CollectionType addAll(
            CollectionType col,
            Iterator<? extends ItemType> items) {
        while (items.hasNext()) {
            col.add(items.next());
        }
        return col;
    }

    public static <CollectionType extends Collection<ItemType>, ItemType> CollectionType addAll(
            CollectionType col,
            ItemType... items) {
        return addAll(col, Arrays.asList(items));
    }

    public static <X> X[] asArray(Class<X> arrayType, Iterable<X> items) {
        return asArray(arrayType, items.iterator());
    }

    @SuppressWarnings("unchecked")
    public static <X> X[] asArray(Class<X> arrayType, Iterator<? extends X> items) {
        List<X> list = asList(items);
        return list.toArray((X[]) Array.newInstance(arrayType, list.size()));
    }

    public static <X> List<X> asList(Iterable<? extends X> items) {
        return addAll(new ArrayList<X>(), items);
    }

    public static <X> List<X> asList(Iterator<? extends X> items) {
        return addAll(new ArrayList<X>(), items);
    }

    public static <X> List<X> asList(X... items) {
        return asList(Arrays.asList(items));
    }

    public static <X> Set<X> asSet(Iterable<? extends X> items) {
        return addAll(new HashSet<X>(), items);
    }

    public static <X> Set<X> asSet(Iterator<? extends X> items) {
        return addAll(new HashSet<X>(), items);
    }

    public static <X> Set<X> asSet(X... items) {
        return asSet(Arrays.asList(items));
    }

    public static <X> Set<X> asOrderedSet(Iterable<? extends X> items) {
        return addAll(new LinkedHashSet<X>(), items);
    }

    public static <X> Set<X> asOrderedSet(Iterator<? extends X> items) {
        return addAll(new LinkedHashSet<X>(), items);
    }

    public static <X> Set<X> asOrderedSet(X... items) {
        return asOrderedSet(Arrays.asList(items));
    }

    /**
     * usefull method if you like to use the new for loop with iterators.
     */
    public static <X> Iterable<X> asIterable(final Iterator<X> items) {
        return new Iterable<X>() {
            @Override
            public Iterator<X> iterator() {
                return items;
            }
        };
    }

    public static <X> Iterable<X> filter(Iterable<X> items, Filter<X, X> filter) {
        return new FilterIterable<X, X>(items, filter);
    }

    public static <X> Iterator<X> filter(Iterator<X> items, Filter<X, X> filter) {
        return new FilterIterator<X, X>(items, filter);
    }

    public static <NewType, OldType> Iterable<NewType> transform(
            Iterable<OldType> itemsToTransform,
            Filter<NewType, OldType> transformer) {
        return new FilterIterable<NewType, OldType>(itemsToTransform, transformer);
    }

    public static <NewType, OldType> Iterator<NewType> transform(
            Iterator<OldType> itemsToTransform,
            Filter<NewType, OldType> transformer) {
        return new FilterIterator<NewType, OldType>(itemsToTransform, transformer);
    }

    public static <Key, Value> Map<Value, Key> mirror(Map<Key, Value> in) {
        return mirror(new HashMap<Value, Key>(), in);
    }

    public static <Key, Value> Map<Value, Key> mirror(Map<Value, Key> target, Map<Key, Value> in) {
        for (Map.Entry<Key, Value> entry : in.entrySet()) {
            target.put(entry.getValue(), entry.getKey());
        }
        return target;
    }

    public static <X> boolean contains(Iterable<X> items, Filter<?, X> filter) {
        return contains(items.iterator(), filter);
    }

    public static <X> boolean contains(Iterator<X> items, Filter<?, X> filter) {
        for (X item; items.hasNext();) {
            item = items.next();
            if (filter.accept(item)) {
                return true;
            }
        }
        return false;
    }

    public static <X> X findFirst(Iterable<X> items, Filter<X, X> filter) {
        return findFirstAndTransform(items, filter);
    }

    public static <X> X findFirst(Iterator<X> items, Filter<X, X> filter) {
        return findFirstAndTransform(items, filter);
    }

    public static <NewType, OldType> NewType findFirstAndTransform(
            Iterable<OldType> items,
            Filter<NewType, OldType> filter) {
        return findFirstAndTransform(items.iterator(), filter);
    }

    public static <NewType, OldType> NewType findFirstAndTransform(
            Iterator<OldType> items,
            Filter<NewType, OldType> filter) {
        for (OldType item; items.hasNext();) {
            item = items.next();
            if (filter.accept(item)) {
                return filter.transform(item);
            }
        }
        return null;
    }

    public static <X> int findIndex(
            Iterable<X> items,
            Filter<?, X> filter) {
        return findIndex(items.iterator(), filter);
    }

    public static <X> int findIndex(
            Iterator<X> items,
            Filter<?, X> filter) {
        for (int i = 0; items.hasNext(); ++i) {
            if (filter.accept(items.next())) {
                return i;
            }
        }
        return -1;
    }
}
