/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.subject;

import java.io.Serializable;
import java.net.URI;

/**
 * A special scope class, whose names are URIs.
 * <p/>
 * Instances of this class can be used to define URN-Names for special principals.
 *
 * @author Marko Reiprecht
 *         Created on 16.09.2007
 * @version $Revision: $
 */
public class URIScope implements Scope, Serializable {

    // ------------------------------ FIELDS ------------------------------

    private static final long serialVersionUID = 3422031289395162664L;

    /**
     * Field m_uri contains the uri, wich describes the name of this scope.
     */
    private URI m_uri;
    /**
     * Field m_scope contains the parent scope.
     */
    private Scope m_scope;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor creates a new URIScope with the specified uri as name.
     * <p/>
     * It throws an IllegalArgumentException if one of the arguments are null.
     *
     * @param uri the uri != null.
     * @throws IllegalArgumentException if the given string is not an uri.
     */
    public URIScope(final String uri) throws IllegalArgumentException {
        this(URI.create(uri), Scope.GLOBAL);
    }

    /**
     * Constructor creates a new URIScope with the specified uri as name.
     * <p/>
     * It throws an IllegalArgumentException if one of the arguments are null.
     *
     * @param uri   the uri != null.
     * @param scope the scope != null.
     * @throws IllegalArgumentException if the given string is not an uri.
     */
    public URIScope(final String uri, final Scope scope) throws IllegalArgumentException {
        this(URI.create(uri), scope);
    }

    /**
     * Constructor creates a new URIScope with the specified uri as name.
     * <p/>
     * It throws an IllegalArgumentException if one of the arguments are null.
     *
     * @param uri the uri != null.
     * @throws IllegalArgumentException if uri is null.
     */
    public URIScope(final URI uri) throws IllegalArgumentException {
        this(uri, Scope.GLOBAL);
    }

    /**
     * Constructor URIScope creates a new URIScope instance.
     * <p/>
     * It throws an IllegalArgumentException if one of the arguments are null.
     *
     * @param uri   the uri != null.
     * @param scope the scope != null.
     * @throws IllegalArgumentException if one of the parameters is null.
     */
    public URIScope(final URI uri, final Scope scope) throws IllegalArgumentException {
        if (uri == null) {
            throw new IllegalArgumentException("the uri of a URIScope must not null");
        }
        if (scope == null) {
            throw new IllegalArgumentException("the scope of a URIScope must not null");
        }

        m_uri = uri;
        m_scope = scope;
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the scope of this URIScope object.
     *
     * @return Scope.
     * @see Scope#getScope()
     */
    public Scope getScope() {
        return m_scope;
    }

    // ------------------------ CANONICAL METHODS ------------------------

    /**
     * Returns the equaltiy.
     *
     * @param o the o.
     * @return boolean.
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        final URIScope uriScope = (URIScope) o;

        if (!m_scope.equals(uriScope.m_scope)) {
            return false;
        }
        if (!m_uri.equals(uriScope.m_uri)) {
            return false;
        }

        return true;
    }

    /**
     * Returns the hashCode.
     *
     * @return int.
     */
    public int hashCode() {
        int result;
        result = m_uri.hashCode();
        result = 31 * result + m_scope.hashCode();
        return result;
    }

    // ------------------------ INTERFACE METHODS ------------------------


    // --------------------- Interface Scope ---------------------

    /**
     * Method gets the name of this URIScope object,
     * this is the string representation of the internal uri.
     *
     * @return String.
     * @see Scope#getName()
     */
    public String getName() {
        return m_uri.toString();
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method gets the URI of this URIScope object.
     *
     * @return URI.
     */
    public URI getURI() {
        return m_uri;
    }


    /**
     * Method toString return the name {@link #getName()}.
     *
     * @return String.
     */
    public String toString() {
        return getName();
    }
}
