package org.n52.security.common.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * The PathWildCardMatch function works like the ant path filter expressions.
 * <ul>
 * <li>'**' means all child paths including '/'</li>
 * <li>'*' means only one child path excluding '/'</li>
 * </ul>
 * <p/>
 * Example: /path/'*'/file matches /path/<childpath>/file<br/>
 * /path/'**'/file matches /path/<childpath>/file or
 * /path/<childpath>/<nextpath>/file<br/>
 * <p/>
 * The match function works case insensitive.
 *
 * @author Marko Reiprecht
 */
public class PathWildcardMatcher {

    private static final Pattern SPECIALCHAR_REGEX = Pattern.compile("(?:([/\\\\]+)|(\\*\\*?))");

    private Pattern m_wildcardPattern;

    public PathWildcardMatcher(final String wildCardString) {
        this(wildCardString, false);
    }

    public PathWildcardMatcher(final String wildCardString, final boolean caseInsensitive) {
        m_wildcardPattern = buildPattern(wildCardString, caseInsensitive);
    }

    /**
     * @return the wildcardPattern
     * @deprecated internal don't use it!
     */
    public Pattern getWildcardPattern() {
        return m_wildcardPattern;
    }

    public boolean matches(final String testString) {
        Pattern p = m_wildcardPattern;
        boolean result = p.matcher(testString != null ? testString : "").matches();
        return result;
    }

    private List<Token> parse(final String wildCardString) {
        if (wildCardString == null || wildCardString.isEmpty()) {
            return Collections.singletonList(eof());
        }
        Matcher m = SPECIALCHAR_REGEX.matcher(wildCardString);
        List<Token> tokens = new ArrayList<Token>();
        int lastEnd = 0;
        while (m.find()) {
            if (m.start() != lastEnd) {
                tokens.add(name(wildCardString.substring(lastEnd, m.start())));
            }
            lastEnd = m.end();
            if (m.group(1) != null) {
                // find Slash or Backslash
                tokens.add(seperator());
            }
            String group2 = m.group(2);
            if (group2 != null) {
                if (group2.length() == 2) {
                    // **
                    tokens.add(doubleAsterisk());
                } else {
                    // *
                    tokens.add(asterisk());
                }
            }
        }
        if (lastEnd != wildCardString.length()) {
            tokens.add(name(wildCardString.substring(lastEnd, wildCardString.length())));
        }
        tokens.add(eof());
        return tokens;
    }

    private Pattern buildPattern(final String wildCardString, final boolean caseInsensitive) {
        List<Token> tokens = parse(wildCardString);
        if (tokens.isEmpty()) {
            return Pattern.compile("^$");
        }
        StringBuilder newRegex = new StringBuilder(wildCardString.length() * 2);
        newRegex.append("^");

        String requiredSeperator = "[/\\\\]+";
        String acceptAll = ".*";

        for (int i = 0, l = tokens.size(); i < l; ++i) {
            Token cur = tokens.get(i);
            TokenType type = cur.getType();
            switch (type) {
                case ASTERISK:
                    // accept all but Slash and Backslash
                    newRegex.append("[^/\\\\]*");
                    break;
                case DOUBLE_ASTERISK:
                    // accept all characters..
                    if (i == 0
                            && tokens.get(i + 1).getType() == TokenType.SEPERATOR) {
                        // '**/' at beginning is fully optional
                        newRegex.append("(?:");
                        newRegex.append(acceptAll);
                        newRegex.append(requiredSeperator);
                        newRegex.append(")?");
                        i += 1;
                        break;
                    }
                    newRegex.append(acceptAll);
                    break;
                case SEPERATOR:
                    if (tokens.get(i + 1).getType() == TokenType.DOUBLE_ASTERISK
                            && tokens.get(i + 2).getType() == TokenType.SEPERATOR) {
                        newRegex.append(acceptAll);
                        i += 1;
                        break;
                    }
                    // accept Slash or Backslash consume repeated
                    newRegex.append(requiredSeperator);
                    break;
                case NAME:
                    newRegex.append("\\Q");
                    newRegex.append(cur.getValue());
                    newRegex.append("\\E");
                    break;
                case EOF:
                    newRegex.append("$");
                    break;
            }
        }
        if (caseInsensitive) {
            return Pattern.compile(newRegex.toString(), Pattern.CASE_INSENSITIVE);
        } else {
            return Pattern.compile(newRegex.toString());
        }
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 37 * hash + (this.m_wildcardPattern != null ? this.m_wildcardPattern.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final PathWildcardMatcher other = (PathWildcardMatcher) obj;
        if (this.m_wildcardPattern != other.m_wildcardPattern && (this.m_wildcardPattern == null || !this.m_wildcardPattern.equals(other.m_wildcardPattern))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return m_wildcardPattern.toString();
    }

    private static Token asterisk() {
        return new Token(TokenType.ASTERISK, "");
    }

    private static Token doubleAsterisk() {
        return new Token(TokenType.DOUBLE_ASTERISK, "");
    }

    private static Token seperator() {
        return new Token(TokenType.SEPERATOR, "");
    }

    private static Token name(String input) {
        if (input == null || input.length() == 0) {
            throw new IllegalArgumentException("name token must have length > 0");
        }
        return new Token(TokenType.NAME, input);
    }

    private static Token eof() {
        return new Token(TokenType.EOF, "");
    }

    private static enum TokenType {

        ASTERISK,
        DOUBLE_ASTERISK,
        SEPERATOR,
        NAME,
        EOF
    }

    private static class Token {

        private TokenType type;
        private String value;

        public Token(TokenType type, String value) {
            this.type = type;
            this.value = value;
        }

        public TokenType getType() {
            return type;
        }

        public String getValue() {
            return value;
        }

    }

}
