/**
 * *****************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Created: 22.03.2008
 *****************************************************************************
 */
package org.n52.security.common.xml;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

/**
 * Tries to load external entities from the classpath.
 * <p/>
 * To find the entities in the classpath a mapping file must be specified. The
 * mapping file is located at "META-INF/xml/entity-mapping.properties".
 * <p/>
 * The file contains entries structured like this:
 * <p/>
 * <code>[publicId|systemId]=path in classpath</code>
 * <p/>
 * e.g.:
 * <p/>
 * <code>http://tempuri.org/test.dtd=META-INF/xml/dtds/test.dtd</code>
 */
public class ClasspathEntityResolver extends ClasspathResourceResolver implements EntityResolver {

    /**
     * Constructor creates a new instance in quiet mode.
     */
    public ClasspathEntityResolver() {
        super();
    }

    /**
     * @param quiet the quiet mode.
     */
    public ClasspathEntityResolver(final boolean quiet) {
        super(quiet);
    }

    /**
     * @param quiet the quiet mode.
     * @param loader the classloader to use, if null
     * this.getClass().getClassLoader() is used.
     */
    public ClasspathEntityResolver(final boolean quiet, final ClassLoader loader) {
        super(quiet, loader);
    }

    /**
     * Method resolves an entity by looking in a special mapping file located at
     * "META-INF/xml/entity-mapping.properties".
     * <p/>
     * First it looks for a publicId mapping and then for a systemId mapping. If
     * none mapping is found it returns a "emtpy" inputsource in quiet mode or
     * null in nonquiet mode.
     *
     * @param publicId the publicId.
     * @param systemId the systemId.
     * @return InputSource.
     */
    @Override
    public InputSource resolveEntity(final String publicId, final String systemId) {
        InputSource resourcefile = lookupIgnoreQuite(systemId);
        if (resourcefile != null) {
            return resourcefile;
        }
        return lookup(publicId);
    }
}
