/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 18.04.2008
 ******************************************************************************/
package org.n52.security.common.xml;

import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

/**
 * Implements the XMLPartialStreamHandler interface with empty method stubs, so that subclasses only need to implement
 * methods in which they are interested.
 * <p/>
 * It declares a helper method, which closes the open xml chunk within the *Befor/AfterChild* methods.
 *
 * @author Marko Reiprecht Created on 18.04.2008
 * @version $Revision: $
 */
public abstract class XMLChunkHandlerAdapter implements XMLChunkHandler {

    /**
     * Method appends the endtag to a open xml chunk.
     *
     * @param xmlChunk
     *            the partialXml.
     * @param endElement
     *            the endElement.
     * @return String.
     */
    protected String closePartialXml(final String xmlChunk, final QName endElement) {
        StringBuffer buffer = new StringBuffer(xmlChunk).append("</");
        if (endElement.getPrefix() != null) {
            buffer.append(endElement.getPrefix()).append(":");
        }
        buffer.append(endElement.getLocalPart()).append(">");
        return buffer.toString();
    }

    /**
     * Gets the current xpath location as simpel element string, without namespaces.
     *
     * @param currentXPath
     * @return a string like /elementA/elementB/..
     */
    protected String getXPathSimple(final List<QName> currentXPath) {
        return getXPath(currentXPath, false, false);
    }

    /**
     * Gets the current xpath as simple string with prefixes.
     *
     * @param currentXPath
     * @return a string like: /x:elementA/y:elementB/..
     */
    protected String getXPathWithPrefixes(final List<QName> currentXPath) {
        return getXPath(currentXPath, true, false);
    }

    /**
     * Gets the current xpath as string with namespaces and prefixes.
     *
     * @param currentXPath
     * @return a string like: /{namespaceX}x:elementA/{namespaceY}y:elementB/..
     */
    protected String getXPathWithNamespacesAndPrefixes(final List<QName> currentXPath) {
        return getXPath(currentXPath, true, true);
    }

    /**
     * Gets the current xpath as string with namespaces.
     *
     * @param currentXPath
     * @return a string like: /{namespaceX}elementA/{namespaceY}elementB/..
     */
    protected String getXPathWithNamespaces(final List<QName> currentXPath) {
        return getXPath(currentXPath, false, true);
    }

    /**
     * simple method building an xpath string from the qnames in the given list.
     *
     * @param currentXPath
     * @param withPrefixes
     *            flag if prefixes included
     * @param withNameSpaces
     *            flag if namespaces included
     * @return a string like /{namespaceX}x:elementA/{namespaceY}y:elementB/..
     */
    private String getXPath(final List<QName> currentXPath, final boolean withPrefixes, final boolean withNameSpaces) {
        StringBuilder xpath = new StringBuilder();
        for (Iterator<QName> xPathIt = currentXPath.iterator(); xPathIt.hasNext();) {
            QName qName = xPathIt.next();
            xpath.append("/");
            if (withNameSpaces) {
                xpath.append("{").append(qName.getNamespaceURI() != null ? qName.getNamespaceURI() : "").append("}");
            }
            if (withPrefixes && qName.getPrefix() != null && qName.getPrefix().length() > 0) {
                xpath.append(qName.getPrefix()).append(":");
            }
            xpath.append(qName.getLocalPart());
        }
        return xpath.toString();
    }

    /**
     * @see de.conterra.wpos.core.xml.XMLStreamChunkHandler#handleChunkAfterChildHandler(java.util.List,
     *      javax.xml.namespace.QName, java.lang.String, javax.xml.namespace.QName,
     *      de.conterra.wpos.core.xml.XMLStreamChunkHandler)
     */
    public void handleChunkAfterChildHandler(final List<QName> currentXPath, final QName handlerQName,
            final String xmlChunk, final QName childQName, final XMLChunkHandler childHandler) {
    }

    /**
     * @see de.conterra.wpos.core.xml.XMLStreamChunkHandler#handleChunkBeforChildHandler(java.util.List,
     *      javax.xml.namespace.QName, java.lang.String, javax.xml.namespace.QName,
     *      de.conterra.wpos.core.xml.XMLStreamChunkHandler)
     */
    public void handleChunkBeforChildHandler(final List<QName> currentXPath, final QName handlerQName,
            final String xmlChunk, final QName childQName, final XMLChunkHandler childHandler) {
    }

    /**
     * @see de.conterra.wpos.core.xml.XMLStreamChunkHandler#handleChunkEnd(java.util.List, javax.xml.namespace.QName,
     *      java.lang.String)
     */
    public void handleChunkEnd(final List<QName> currentXPath, final QName handlerQName, final String xmlChunk) {
    }

    /**
     * @see de.conterra.wpos.core.xml.XMLStreamChunkHandler#handleChunkStart(java.util.List, javax.xml.namespace.QName)
     */
    public void handleChunkStart(final List<QName> currentXPath, final QName handlerQName) {
    }
}
